/*!
 * rollup-plugin-vue2 v0.1.2
 * (c) 2016 Thomas Ghysels <info@thomasg.be>
 * Release under the MIT License.
 */
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var rollupPluginutils = require('rollup-pluginutils');
var parse5 = _interopDefault(require('parse5'));
var compiler = _interopDefault(require('vue-template-compiler'));

function log (text) {
  console.log(text)
}

function warn (text) {
  console.log(red(text))
}



function green (text) {
  return '\u001b[1m\u001b[32m' + text + '\u001b[39m\u001b[22m'
}

function red (text) {
  return '\u001b[1m\u001b[31m' + text + '\u001b[39m\u001b[22m'
}

function getSize (bytes) {
  return bytes < 10000
    ? bytes.toFixed(0) + ' B'
    : bytes < 1024000
    ? (bytes / 1024).toPrecision(3) + ' kB'
    : (bytes / 1024 / 1024).toPrecision(4) + ' MB'
}

/**
 * Wrap a piece of code in a function
 *
 * @param {String} code
 * @return {String}
 */
function toFunction (code) {
  return 'function(){' + code + '}'
}

/**
 * Only support for es5 modules
 *
 * @param script
 * @param template
 * @returns {string}
 */
function injectTemplate (script, template) {
  // Compile template
  var compiled = compiler.compile(template)
  var moduleFuncs = '\nrender: ' + toFunction(compiled.render) + ',' +
    '\nstaticRenderFns: [' + compiled.staticRenderFns.map(toFunction).join(',') + ']'

  // There is no script tag
  if (!script || script.length < 20) {
    return 'export default {' + moduleFuncs + '\n}'
  }

  // Inject in actual script
  var matches = /(export default[^{]*\{)/g.exec(script)
  if (!matches) {
    throw new Error('[rollup-plugin-vue2] could not find place to inject template in script.')
  }
  return script.split(matches[1]).join(matches[1] + moduleFuncs + ',')
}

/**
 * Check the lang attribute of a parse5 node.
 *
 * @param {Node} node
 * @return {String|undefined}
 */
function checkLang (node) {
  if (node.attrs) {
    var i = node.attrs.length
    while (i--) {
      var attr = node.attrs[i]
      if (attr.name === 'lang') {
        return attr.value
      }
    }
  }
  return undefined
}

function vueTransform (code, id) {
  // Parse the file into an HTML tree
  var fragment = parse5.parseFragment(code)

  // Walk through the top level nodes and check for their types
  var nodes = {}
  for (var i = fragment.childNodes.length - 1; i >= 0; i--) {
    nodes[fragment.childNodes[i].nodeName] = fragment.childNodes[i]
  }

  // Process script
  var js = nodes.script && parse5.serialize(nodes.script) || 'export default {\n}'

  // Precompile and inject Vue template
  if (nodes.template && nodes.template.content) {
    var template = parse5.serialize(nodes.template.content)
    template = template.replace(/&amp;&amp;/g, '&&')
    js = injectTemplate(js, template)
    if (template.errors) {
      log(id)
      template.errors.forEach(warn)
    }
  }

  // Import css as a module
  // Example: import "./src/App.vue.component.css"
  var css
  if (nodes.style) {
    var lang = checkLang(nodes.style) || 'css'
    js = 'import ' + JSON.stringify(id + '.component.' + lang) + '\n' + js
    css = parse5.serialize(nodes.style)
  }

  return { js: js, css: css }
}

function vue2 (options) {
  if ( options === void 0 ) options = {};

  var filter = rollupPluginutils.createFilter(options.include, options.exclude)
  var styles = {}

  return {
    name: 'vue2',
    options: function options$1 (options) {
      options.useStrict = false
      return options
    },
    resolveId: function resolveId (id) {
      if (id.indexOf('.vue.component.') !== -1) {
        return id
      }
    },
    load: function load (id) {
      if (id.indexOf('.vue.component.') !== -1) {
        id = id.slice(0, id.lastIndexOf('.vue.component.') + 4)
        return styles[id] || ''
      }
    },
    transform: function transform (source, id) {
      if (!filter(id) || !id.endsWith('.vue')) {
        return
      }

      var ref = vueTransform(source, id);
      var js = ref.js;
      var css = ref.css;

      // Map of every stylesheet
      styles[id] = css

      // Replace "with(this){" with something that works in strict mode
      // https://github.com/vuejs/vue-template-es2015-compiler/blob/master/index.js
      return js.replace(/with\(this\)/g, 'if(window.__VUE_WITH__)')
    },
    transformBundle: function transformBundle (code) {
      return code
        .replace(/process\.env\.VUE_ENV/g, JSON.stringify(process.env.VUE_ENV || ''))
        .replace(/process\.env\.NODE_ENV/g, JSON.stringify(process.env.NODE_ENV || ''))
    },
    ongenerate: function ongenerate (opts, rendered) {
      // Revert "with(this){"
      rendered.code = rendered.code.replace(/if\s*\(window.__VUE_WITH__\)/g, 'with(this)')
    }
  }
}

module.exports = vue2;
